#include <algorithm>
#include <iostream>
#include <assert.h>
#include <vector>
#include <random>

using namespace std;

mt19937 rnd(22);
const int MAXN = 1e5 + 5;

int parent[MAXN];
vector <int> tAdj[MAXN], gAdj[MAXN];

void dfs(int x, int last)
{
    parent[x] = last;
    for(int y: tAdj[x])
    {
        if(y==last) continue;
        dfs(y, x);
    }
}

bool checkAnswer(int N, int V, const vector <int> &c)
{
    for(int x = 0;x<V;x++)
    {
        for(int y: gAdj[x])
        {
            if(y<x) continue;
            if(binary_search(tAdj[c[y]].begin(), tAdj[c[y]].end(), c[x])==false) return false;
        }
    }

    return true;
}

std::vector<int> solve(int N, int V, int E,
                       vector<int> A, vector<int> B,
                       vector<int> X, vector<int> Y)
{
    for(int x = 0;x<V;x++)
    {
        tAdj[x].clear();
        gAdj[x].clear();
    }

    vector <int> c(V, -1);
    for(int i = 0;i<N;i++) c[i] = i;

    for(int i = 0;i<N-1;i++)
    {
        tAdj[A[i]].push_back(B[i]);
        tAdj[B[i]].push_back(A[i]);
    }
    for(int i = 0;i<E;i++)
    {
        gAdj[X[i]].push_back(Y[i]);
        gAdj[Y[i]].push_back(X[i]);
    }

    for(int x = 0;x<N;x++) sort(tAdj[x].begin(), tAdj[x].end());
    for(int x = 0;x<V;x++) sort(gAdj[x].begin(), gAdj[x].end());
    dfs(0, -1);

    vector <int> order;
    for(int x = 0;x<V;x++) order.push_back(x);

    do
    {
        shuffle(order.begin(), order.end(), rnd);

        while(true)
        {
            bool update = false;
            for(int x: order)
            {
                if(c[x]!=-1) continue;

                int root = -1, other = -1;
                for(int y: gAdj[x])
                {
                    if(c[y]==0) root = 0;
                    else if(c[y]!=-1) other = c[y];
                }

                if(other!=-1)
                {
                    c[x] = parent[other];
                    update = true;
                }
                else if(root!=-1)
                {
                    c[x] = tAdj[0][rnd()%int(tAdj[0].size())];
                    update = true;
                }
            }

            if(update==false)
            {
                for(int x = 0;x<V;x++)
                {
                    if(c[x]==-1)
                    {
                        c[x] = 0;
                        update = true;

                        break;
                    }
                }
            }
            if(update==false) break;
        }
    }
    while(checkAnswer(N, V, c)==false);

    return c;
}

#ifdef LOCAL

int main()
{
    vector <int> v;

    //v = solve(2, 4, 2, {0}, {1}, {2, 3}, {0, 1});
    //v = solve(4, 5, 3, {0, 0, 0}, {1, 2, 3}, {1, 2, 3}, {4, 4, 4});
    v = solve(3, 4, 2, {0, 1}, {1, 2}, {3, 3}, {0, 2});

    for(int x: v) cout << x << " ";
    cout << '\n';
}

#endif // LOCAL
